# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from typing import Final

ONC_RPC_PROCEDURE_NAME: Final = "onc_rpc.procedure.name"
"""
ONC/Sun RPC procedure name.
"""

ONC_RPC_PROCEDURE_NUMBER: Final = "onc_rpc.procedure.number"
"""
ONC/Sun RPC procedure number.
"""

ONC_RPC_PROGRAM_NAME: Final = "onc_rpc.program.name"
"""
ONC/Sun RPC program name.
"""

ONC_RPC_VERSION: Final = "onc_rpc.version"
"""
ONC/Sun RPC program version.
"""
