# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from typing import Final

ORACLE_CLOUD_REALM: Final = "oracle_cloud.realm"
"""
The OCI realm identifier that indicates the isolated partition in which the tenancy and its resources reside.
Note: See [OCI documentation on realms](https://docs.oracle.com/iaas/Content/General/Concepts/regions.htm).
"""
