# Do not edit this file.
# This file is generated automatically by executing tox -e generate-workflows

name: Lint

on:
  workflow_call:

permissions:
  contents: read

env:
  CORE_REPO_SHA: main
  # Set the SHA to the branch name if the PR has a label 'prepare-release' or 'backport' otherwise, set it to 'main'
  # For PRs you can change the inner fallback ('main')
  # For pushes you change the outer fallback ('main')
  # The logic below is used during releases and depends on having an equivalent branch name in the contrib repo.
  CONTRIB_REPO_SHA: {% raw %}${{ github.event_name == 'pull_request' && (
       contains(github.event.pull_request.labels.*.name, 'prepare-release') && github.event.pull_request.head.ref ||
       contains(github.event.pull_request.labels.*.name, 'backport') && github.event.pull_request.base.ref ||
       'main'
    ) || 'main' }}{% endraw %}
  PIP_EXISTS_ACTION: w

jobs:
  {%- for job_data in job_datas %}

  {{ job_data.name }}:
    name: {{ job_data.ui_name }}
    runs-on: ubuntu-latest
    timeout-minutes: 30
    steps:
      - name: Checkout repo @ SHA - ${% raw %}{{ github.sha }}{% endraw %}
        uses: actions/checkout@v4

      - name: Set up Python 3.14
        uses: actions/setup-python@v5
        with:
          python-version: "3.14"

      - name: Install tox
        run: pip install tox-uv

      - name: Run tests
        run: tox -e {{ job_data.tox_env }}
  {%- endfor %}
